/*
   lock.c -- PIKT piktc & piktc_svc lock routines

   $Header: /var/cvs/PIKT/pikt/src/piktc/lock.c,v 1.22 2007/01/18 13:23:39 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2007 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#include "../lib/common.h"
#include "piktc.h"

int islock(char *);
int mklock(char *, char *, int, int);
int rmlock(char *, int, int);
int rmlocks(int, int);

/* return TRUE if lockfile exists, FALSE if not
 */
int
islock (char *lockfile)
{
	struct stat fbuf;

	if (! lockfile)
		return FALSE;
	if (lstat(lockfile, &fbuf))
		return FALSE;
	else
		return TRUE;
}

/* open a lock file, write contents if any, then close
 */
int
mklock (char *lockfile, char *contents, int interact, int log)
{
	static char fnnam[] = "mklock";
	FILE *lkfil;

	if (! lockfile)
		return ERROR;
	if ((lkfil = fopen(lockfile, "w")) == NULL) {
		if (interact) {
			logreturn(OUTPUT, LOG_ERR, ERRID_FILE_CREATE_ERROR, __FILE__, __LINE__, fnnam, TRUE, "unable to create %s file!", lockfile);
		}
		if (log)
			logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_OPEN_ERROR, __FILE__, __LINE__, fnnam, TRUE, "fopen() error for %s", lockfile);
		return ERROR;
	}
	if (contents)
		fprintf(lkfil, "%s\n", contents);
	if (fclose(lkfil)) {
		if (interact) {
			logreturn(OUTPUT, LOG_ERR, ERRID_FILE_CREATE_ERROR, __FILE__, __LINE__, fnnam, TRUE, "unable to create %s file!", lockfile);
		}
		if (log)
			logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_CLOSE_ERROR, __FILE__, __LINE__, fnnam, TRUE,  "fclose() error for %s", lockfile);
		return ERROR;
	}
	chmod(lockfile, (mode_t)(S_IRUSR|S_IWUSR|S_IRGRP));	/* 640 */
	return 0;
}

/* remove a lock file
 */
int
rmlock (char *lockfile, int interact, int log)
{
	static char fnnam[] = "rmlock";

	if (! lockfile)
		return 0;
	if (remove(lockfile)) {		/* remove returns 0 on success */
		if (interact) {
			logreturn(OUTPUT, LOG_ERR, ERRID_FILE_REMOVE_ERROR, __FILE__, __LINE__, fnnam, TRUE, "unable to remove %s file!", lockfile);
		}
		if (log)
			logreturn(PIKTLOG, LOG_ERR, ERRID_FILE_REMOVE_ERROR, __FILE__, __LINE__, fnnam, TRUE, "remove() error for %s", lockfile);
		return ERROR;
	}
	return 0;
}

/* remove all pertinent locks
 */
int
rmlocks (int interact, int log)
{
	int res;

	res = 0;
	#ifdef PIKTC_SVC
	if (islock(piktc_svclock)) {
		if (rmlock(piktc_svclock, FALSE, log) == ERROR)
			res = ERROR;
	}
	#else
	if (islock(piktclock)) {
		if (rmlock(piktclock, interact, log) == ERROR)
			res = ERROR;
	}
	#endif
	return res;
}
